import sys
from unittest import mock

from wayback_machine_archiver.archiver import main, format_archive_url


@mock.patch("wayback_machine_archiver.archiver.LegacyClient")
@mock.patch("wayback_machine_archiver.archiver.os.getenv", return_value=None)
@mock.patch("wayback_machine_archiver.archiver.random.shuffle")
def test_random_order_flag_shuffles_urls(mock_shuffle, mock_getenv, mock_legacy_client):
    """
    Verify that when --random-order is passed, random.shuffle is called
    and the client's archive method is called for each URL.
    """
    # Arrange: Simulate command-line arguments
    urls_to_archive = ["http://test.com/a", "http://test.com/b"]
    sys.argv = ["archiver", "--random-order"] + urls_to_archive

    # Get a reference to the mock client instance that main() will create
    mock_client_instance = mock_legacy_client.return_value

    # Act: Run the main function
    main()

    # Assert: Check that our mocks were called as expected
    mock_shuffle.assert_called_once()
    assert mock_client_instance.archive.call_count == len(urls_to_archive)

    # Verify the content of the calls is correct
    called_urls = {call.args[0] for call in mock_client_instance.archive.call_args_list}
    expected_urls = {format_archive_url(u) for u in urls_to_archive}
    assert called_urls == expected_urls


@mock.patch("wayback_machine_archiver.archiver.LegacyClient")
@mock.patch("wayback_machine_archiver.archiver.os.getenv", return_value=None)
@mock.patch("wayback_machine_archiver.archiver.random.shuffle")
def test_default_order_does_not_shuffle(mock_shuffle, mock_getenv, mock_legacy_client):
    """
    Verify that without --random-order, shuffle is not called,
    but the client's archive method is still called for each URL.
    """
    # Arrange: Simulate command-line arguments
    urls_to_archive = ["http://test.com/a", "http://test.com/b"]
    sys.argv = ["archiver"] + urls_to_archive

    mock_client_instance = mock_legacy_client.return_value

    # Act: Run the main function
    main()

    # Assert: Check that shuffle was NOT called, but the client was.
    mock_shuffle.assert_not_called()
    assert mock_client_instance.archive.call_count == len(urls_to_archive)

    # Verify the content of the calls is correct
    called_urls = {call.args[0] for call in mock_client_instance.archive.call_args_list}
    expected_urls = {format_archive_url(u) for u in urls_to_archive}
    assert called_urls == expected_urls



import logging
import time
import requests


class LegacyClient:
    """Handles archiving using the public, unauthenticated API."""

    def __init__(self, session):
        self.session = session

    def archive(self, request_url, rate_limit_wait):
        """Submit a url to the Internet Archive to archive."""
        if rate_limit_wait > 0:
            logging.debug("Sleeping for %s seconds", rate_limit_wait)
            time.sleep(rate_limit_wait)
        logging.info("Calling archive url %s", request_url)
        r = self.session.head(request_url, allow_redirects=True)
        try:
            # Raise `requests.exceptions.HTTPError` if 4XX or 5XX status
            r.raise_for_status()
        except requests.exceptions.HTTPError as e:
            logging.exception(e)
            raise


class SPN2Client:
    """Handles archiving using the authenticated SPN2 API."""

    SAVE_URL = "https://web.archive.org/save"
    STATUS_URL_TEMPLATE = "https://web.archive.org/save/status/{job_id}"

    def __init__(self, session, access_key, secret_key):
        self.session = session
        auth_header = f"LOW {access_key}:{secret_key}"
        self.session.headers.update(
            {
                "Authorization": auth_header,
                "Accept": "application/json",
            }
        )

    def submit_capture(self, url_to_archive, rate_limit_wait):
        """
        Submits a capture request to the SPN2 API.
        Returns the job_id for the capture request.
        """
        if rate_limit_wait > 0:
            logging.debug("Sleeping for %s seconds", rate_limit_wait)
            time.sleep(rate_limit_wait)
        logging.info("Submitting %s to SPN2", url_to_archive)
        data = {"url": url_to_archive}
        try:
            r = self.session.post(self.SAVE_URL, data=data)
            r.raise_for_status()
            response_json = r.json()
            job_id = response_json.get("job_id")
            logging.info(
                "Successfully submitted %s, job_id: %s", url_to_archive, job_id
            )
            return job_id
        except requests.exceptions.HTTPError as e:
            logging.error("HTTP Error submitting URL %s: %s", url_to_archive, e)
            logging.error("Response content: %s", e.response.text)
            raise
        except Exception as e:
            logging.exception(e)
            raise

    def check_status(self, job_id):
        """
        Checks the status of a capture job.
        Returns the JSON response from the status API.
        """
        status_url = self.STATUS_URL_TEMPLATE.format(job_id=job_id)
        logging.debug("Checking status for job_id: %s", job_id)
        try:
            r = self.session.get(status_url)
            r.raise_for_status()
            return r.json()
        except requests.exceptions.HTTPError as e:
            logging.error("HTTP Error checking status for job %s: %s", job_id, e)
            logging.error("Response content: %s", e.response.text)
            raise
        except Exception as e:
            logging.exception(e)
            raise



import pytest
from wayback_machine_archiver.clients import LegacyClient
from requests.adapters import HTTPAdapter
import requests


@pytest.fixture
def session():
    session = requests.Session()
    session.mount("https://", HTTPAdapter())
    session.mount("http://", HTTPAdapter())
    return session


def test_legacy_client_archive(requests_mock, session):
    url = "https://web.archive.org/save/yahoo.com"
    requests_mock.head(url)
    client = LegacyClient(session=session)
    client.archive(url, rate_limit_wait=0)
    assert True


def test_legacy_client_archive_with_404(requests_mock, session):
    url = "https://web.archive.org/save/yahoo.com"
    requests_mock.head(url, status_code=404)
    with pytest.raises(requests.exceptions.HTTPError):
        client = LegacyClient(session=session)
        client.archive(url, rate_limit_wait=0)



from requests.adapters import HTTPAdapter
from urllib3.util.retry import Retry
import argparse
import logging
import os
import random
import re
import requests
import time
import xml.etree.ElementTree as ET
from dotenv import load_dotenv
from .clients import LegacyClient, SPN2Client

# Library version
__version__ = "2.2.0"


# String used to prefix local sitemaps
LOCAL_PREFIX = "file://"


def format_archive_url(url):
    """Given a URL, constructs an Archive URL to submit the archive request."""
    logging.debug("Creating archive URL for %s", url)
    SAVE_URL = "https://web.archive.org/save/"
    request_url = SAVE_URL + url

    return request_url


def get_namespace(element):
    """Extract the namespace using a regular expression."""
    match = re.match(r"\{.*\}", element.tag)
    return match.group(0) if match else ""


def download_remote_sitemap(sitemap_url, session):
    """Download the sitemap of the target website."""
    logging.debug("Downloading: %s", sitemap_url)
    r = session.get(sitemap_url)
    try:
        # Raise `requests.exceptions.HTTPError` if 4XX or 5XX status
        r.raise_for_status()
    except requests.exceptions.HTTPError as e:
        logging.exception(e)
        raise
    else:
        return r.text.encode("utf-8")


def load_local_sitemap(sitemap_filepath):
    """Load a local sitemap and return it as a string."""
    logging.debug("Loading local sitemap: %s", sitemap_filepath)

    if sitemap_filepath.startswith(LOCAL_PREFIX):
        sitemap_filepath = sitemap_filepath[len(LOCAL_PREFIX) :]

    # Try to open the file, error on failure
    try:
        logging.debug("Opening local file '%s'", sitemap_filepath)
        with open(sitemap_filepath, "r") as fp:
            contents = fp.read()
    except IOError as e:
        logging.exception(e)
        raise

    return contents


def sitemap_is_local(sitemap_url):
    """Returns True if we believe a URI to be local, False otherwise."""
    return sitemap_url.startswith(LOCAL_PREFIX) or sitemap_url.startswith("/")


def extract_pages_from_sitemap(site_map_text):
    """Extract the various pages from the sitemap text."""
    root = ET.fromstring(site_map_text)

    # Sitemaps use a namespace in the XML, which we need to read
    namespace = get_namespace(root)

    urls = []
    for loc_node in root.findall(".//{}loc".format(namespace)):
        urls.append(loc_node.text)

    return set(urls)


def _submit_next_url(
    urls_to_process,
    client,
    pending_jobs,
    rate_limit_in_sec,
    submission_attempts,
    max_retries=3,
):
    """
    Pops the next URL, submits it, and adds its job_id to pending_jobs.
    If submission fails, it re-queues the URL up to max_retries.
    """
    url = urls_to_process.pop(0)
    attempt_num = submission_attempts.get(url, 0) + 1
    submission_attempts[url] = attempt_num

    if attempt_num > max_retries:
        logging.error("URL %s failed submission %d times, giving up.", url, max_retries)
        return

    try:
        logging.info("Submitting %s (attempt %d/%d)...", url, attempt_num, max_retries)
        job_id = client.submit_capture(url, rate_limit_wait=rate_limit_in_sec)
        if job_id:
            pending_jobs[job_id] = url
            # On success, we can remove it from the attempts tracker
            if url in submission_attempts:
                del submission_attempts[url]
    except Exception as e:
        logging.warning(
            "Failed to submit URL %s: %s. Re-queuing for another attempt.", url, e
        )
        urls_to_process.append(url)


def _poll_pending_jobs(client, pending_jobs, poll_interval_sec=0.2):
    """
    Checks the status of pending jobs, removing completed ones.
    Waits a short interval between each check to avoid slamming the API.
    """
    for job_id in list(pending_jobs.keys()):
        original_url = pending_jobs[job_id]
        try:
            status_data = client.check_status(job_id)
            status = status_data.get("status")

            if status == "success":
                timestamp = status_data.get("timestamp")
                archive_url = f"https://web.archive.org/web/{timestamp}/{original_url}"
                logging.info("Success for job %s: %s", job_id, archive_url)
                del pending_jobs[job_id]
            elif status == "error":
                message = status_data.get("message", "Unknown error")
                logging.error(
                    "Error for job %s (%s): %s", job_id, original_url, message
                )
                del pending_jobs[job_id]
            else:
                logging.debug("Job %s (%s) is pending...", job_id, original_url)

        except Exception as e:
            logging.error(
                "An exception occurred while checking job %s (%s): %s",
                job_id,
                original_url,
                e,
            )
            del pending_jobs[job_id]

        # Be nice to the polling API
        time.sleep(poll_interval_sec)


def main():
    # Command line parsing
    parser = argparse.ArgumentParser(
        prog="archiver",
        description="A script to backup a web pages with Internet Archive",
    )
    parser.add_argument(
        "--version",
        action="version",
        version="%(prog)s {version}".format(version=__version__),
    )
    parser.add_argument(
        "urls",
        nargs="*",
        default=[],
        help="the URLs of the pages to archive",
    )
    parser.add_argument(
        "--file",
        help="path to a file containing urls to save (one url per line)",
        required=False,
    )
    parser.add_argument(
        "--sitemaps",
        nargs="+",
        default=[],
        help="one or more URIs to sitemaps listing pages to archive; local paths must be prefixed with '{f}'".format(
            f=LOCAL_PREFIX
        ),
        required=False,
    )
    parser.add_argument(
        "--log",
        help="set the logging level, defaults to WARNING (case-insensitive)",
        dest="log_level",
        default=logging.WARNING,
        type=str.upper,
        choices=[
            "DEBUG",
            "INFO",
            "WARNING",
            "ERROR",
            "CRITICAL",
        ],
    )
    parser.add_argument(
        "--log-to-file",
        help="redirect logs to a file",
        dest="log_file",
        default=None,
    )
    parser.add_argument(
        "--archive-sitemap-also",
        help="also submit the URL of the sitemap to be archived",
        dest="archive_sitemap",
        default=False,
        action="store_true",
    )
    parser.add_argument(
        "--rate-limit-wait",
        help="number of seconds to wait between page requests to avoid flooding the archive site, defaults to 15",
        dest="rate_limit_in_sec",
        default=15,
        type=int,
    )
    parser.add_argument(
        "--random-order",
        help="randomize the order of pages before archiving",
        dest="random_order",
        default=False,
        action="store_true",
    )

    args = parser.parse_args()

    # Set the logging level based on the arguments
    logging.basicConfig(level=args.log_level, filename=args.log_file)

    # This will load variables from a .env file into the environment.
    # It will NOT overwrite any existing environment variables.
    load_dotenv()

    access_key = os.getenv("INTERNET_ARCHIVE_ACCESS_KEY")
    secret_key = os.getenv("INTERNET_ARCHIVE_SECRET_KEY")
    use_spn2 = access_key and secret_key

    # Enforce API minimums for both modes
    if use_spn2:
        MIN_SPN2_WAIT_SEC = 5
        if args.rate_limit_in_sec < MIN_SPN2_WAIT_SEC:
            logging.warning(
                "Provided rate limit of %d seconds is below the API minimum of %d for authenticated users. Overriding to %d seconds to avoid rate-limiting.",
                args.rate_limit_in_sec,
                MIN_SPN2_WAIT_SEC,
                MIN_SPN2_WAIT_SEC,
            )
            args.rate_limit_in_sec = MIN_SPN2_WAIT_SEC
    else:
        MIN_LEGACY_WAIT_SEC = 15
        if args.rate_limit_in_sec < MIN_LEGACY_WAIT_SEC:
            logging.warning(
                "Provided rate limit of %d seconds is below the API minimum of %d for unauthenticated users. Overriding to %d seconds to avoid rate-limiting.",
                args.rate_limit_in_sec,
                MIN_LEGACY_WAIT_SEC,
                MIN_LEGACY_WAIT_SEC,
            )
            args.rate_limit_in_sec = MIN_LEGACY_WAIT_SEC

    logging.debug("Archiver Version: %s", __version__)
    logging.debug("Arguments: %s", args)

    urls_to_archive = []
    # Add the regular pages
    if args.urls:
        logging.info("Adding page URLs to archive")
        logging.debug("Page URLs to archive: %s", args.urls)
        urls_to_archive.extend(args.urls)

    # Set up retry and backoff
    session = requests.Session()
    session.max_redirects = 100

    retries = Retry(
        total=5,
        backoff_factor=args.rate_limit_in_sec,
        status_forcelist=[500, 502, 503, 504, 520],
    )

    session.mount("https://", HTTPAdapter(max_retries=retries))
    session.mount("http://", HTTPAdapter(max_retries=retries))

    # Download and process the sitemaps
    remote_sitemaps = set()
    logging.info("Parsing sitemaps")
    for sitemap_url in args.sitemaps:
        # Save the remote ones, incase the user wants us to backthem up
        if sitemap_is_local(sitemap_url):
            logging.debug("The sitemap '%s' is local.", sitemap_url)
            sitemap_xml = load_local_sitemap(sitemap_url)
        else:
            logging.debug("The sitemap '%s' is remote.", sitemap_url)
            if args.archive_sitemap:
                remote_sitemaps.add(sitemap_url)
            sitemap_xml = download_remote_sitemap(sitemap_url, session=session)

        urls_to_archive.extend(extract_pages_from_sitemap(sitemap_xml))

    # Archive the sitemap as well, if requested
    if args.archive_sitemap:
        logging.info("Archiving sitemaps")
        if remote_sitemaps:
            urls_to_archive.extend(remote_sitemaps)
        else:
            logging.debug("No remote sitemaps to backup.")

    # And URLs from file
    if args.file:
        logging.info("Reading urls from file: %s", args.file)
        with open(args.file) as file:
            urls_from_file = (u.strip() for u in file.readlines() if u.strip())
            urls_to_archive.extend(urls_from_file)

    # Deduplicate URLs and convert to a list
    urls_to_archive_list = list(set(urls_to_archive))

    # Randomize the order if requested
    if args.random_order:
        logging.info("Randomizing the order of URLs.")
        random.shuffle(urls_to_archive_list)

    # Archive the URLs
    logging.debug("Archive URLs: %s", urls_to_archive_list)

    if use_spn2:
        logging.info("SPN2 credentials found. Using authenticated API.")
        client = SPN2Client(
            session=session, access_key=access_key, secret_key=secret_key
        )

        urls_to_process = list(urls_to_archive_list)
        pending_jobs = {}
        submission_attempts = {}  # Tracks failed submission retries

        logging.info(
            "Beginning interleaved submission and polling of %d URLs...",
            len(urls_to_process),
        )

        while urls_to_process or pending_jobs:
            if urls_to_process:
                _submit_next_url(
                    urls_to_process,
                    client,
                    pending_jobs,
                    args.rate_limit_in_sec,
                    submission_attempts,
                )

            if pending_jobs:
                _poll_pending_jobs(client, pending_jobs)

            if not urls_to_process and pending_jobs:
                wait_time = 5
                logging.info(
                    "%d captures remaining, starting next polling cycle in %d seconds...",
                    len(pending_jobs),
                    wait_time,
                )
                time.sleep(wait_time)

        logging.info("All captures complete.")

    else:
        logging.warning("No SPN2 credentials found. Using public, unauthenticated API.")
        client = LegacyClient(session=session)
        # For legacy, we must format the URL first
        archive_urls_list = list(map(format_archive_url, urls_to_archive_list))

        logging.info("Archiving %d URLs sequentially...", len(archive_urls_list))
        for url in archive_urls_list:
            try:
                client.archive(url, rate_limit_wait=args.rate_limit_in_sec)
            except Exception as e:
                logging.error("Failed to archive URL %s: %s", url, e)


if __name__ == "__main__":
    main()



import pytest
from wayback_machine_archiver.archiver import download_remote_sitemap
from requests.adapters import HTTPAdapter
import requests


SITEMAP = """<?xml version="1.0" encoding="UTF-8"?>
    <urlset xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xsi:schemaLocation="http://www.sitemaps.org/schemas/sitemap/0.9 http://www.sitemaps.org/schemas/sitemap/0.9/sitemap.xsd" xmlns="http://www.sitemaps.org/schemas/sitemap/0.9">
    <url>
    <loc>https://alexgude.com/blog/double-checking-538/</loc>
    <lastmod>2016-04-28T00:00:00+00:00</lastmod>
    </url>
    <url>
    <loc>https://alexgude.com/files/undergrad_thesis.pdf</loc>
    <lastmod>2019-05-09T16:19:45+00:00</lastmod>
    </url>
    </urlset>
"""


@pytest.fixture
def session():
    session = requests.Session()
    session.mount("https://", HTTPAdapter())
    session.mount("http://", HTTPAdapter())
    return session


def test_download_remote_sitemap(requests_mock, session):
    url = "https://www.radiokeysmusic.com/sitemap.xml"
    requests_mock.get(url, text=SITEMAP)
    returned_contents = download_remote_sitemap(url, session)
    assert returned_contents == SITEMAP.encode("UTF-8")


def test_download_remote_sitemap_with_status_error(requests_mock, session):
    url = "https://www.radiokeysmusic.com/sitemap.xml"
    requests_mock.get(url, text=SITEMAP, status_code=404)
    with pytest.raises(requests.exceptions.HTTPError):
        download_remote_sitemap(url, session)



from unittest import mock
import pytest
from wayback_machine_archiver.archiver import _submit_next_url, _poll_pending_jobs

# --- Tests for _submit_next_url ---


def test_submit_next_url_success():
    """
    Verify that a successful submission adds the job_id to the pending_jobs
    dictionary, consumes the URL, and clears the attempts tracker for that URL.
    """
    mock_client = mock.Mock()
    mock_client.submit_capture.return_value = "job-123"

    urls_to_process = ["http://example.com"]
    pending_jobs = {}
    # Simulate a previous failure to ensure the tracker is cleared on success
    submission_attempts = {"http://example.com": 1}

    _submit_next_url(urls_to_process, mock_client, pending_jobs, 5, submission_attempts)

    # Assertions
    mock_client.submit_capture.assert_called_once_with(
        "http://example.com", rate_limit_wait=5
    )
    assert pending_jobs == {"job-123": "http://example.com"}
    assert not urls_to_process, "URL should have been consumed from the list"
    assert "http://example.com" not in submission_attempts, (
        "Attempts tracker should be cleared on success"
    )


def test_submit_next_url_failure_requeues_and_tracks_attempt():
    """
    Verify that a failed submission re-queues the URL at the end of the list
    and increments its attempt count.
    """
    mock_client = mock.Mock()
    mock_client.submit_capture.side_effect = Exception("API Error")

    urls_to_process = ["http://a.com", "http://b.com"]
    pending_jobs = {}
    submission_attempts = {}

    _submit_next_url(urls_to_process, mock_client, pending_jobs, 5, submission_attempts)

    # Assertions
    assert not pending_jobs, "No job should have been added on failure"
    assert urls_to_process == ["http://b.com", "http://a.com"], (
        "Failed URL should be at the end of the list"
    )
    assert submission_attempts == {"http://a.com": 1}, (
        "Attempt count should be incremented"
    )


def test_submit_next_url_gives_up_after_max_retries():
    """
    Verify that if a URL has reached its max retry count, it is not
    re-queued and the submission is not attempted.
    """
    mock_client = mock.Mock()

    urls_to_process = ["http://will-fail.com"]
    pending_jobs = {}
    # Simulate that the URL has already failed 3 times
    submission_attempts = {"http://will-fail.com": 3}

    _submit_next_url(
        urls_to_process,
        mock_client,
        pending_jobs,
        5,
        submission_attempts,
        max_retries=3,
    )

    # Assertions
    mock_client.submit_capture.assert_not_called()
    assert not pending_jobs
    assert not urls_to_process, "URL should be consumed but not re-queued"
    assert submission_attempts == {"http://will-fail.com": 4}, (
        "Attempt count is still updated"
    )


# --- Tests for _poll_pending_jobs ---


@mock.patch("wayback_machine_archiver.archiver.time.sleep")
def test_poll_removes_completed_jobs(mock_sleep):
    """
    Verify that jobs with 'success' or 'error' status are removed from the
    pending list, while 'pending' jobs remain.
    """
    mock_client = mock.Mock()
    # Define the return values for consecutive calls to check_status
    mock_client.check_status.side_effect = [
        {"status": "success", "original_url": "http://a.com", "timestamp": "20250101"},
        {"status": "error", "message": "Too many redirects."},
        {"status": "pending"},
    ]

    pending_jobs = {
        "job-success": "http://a.com",
        "job-error": "http://b.com",
        "job-pending": "http://c.com",
    }

    _poll_pending_jobs(mock_client, pending_jobs)

    # Assertions
    assert pending_jobs == {"job-pending": "http://c.com"}, (
        "Only the pending job should remain"
    )
    assert mock_client.check_status.call_count == 3
    assert mock_sleep.call_count == 3, "Should sleep after each poll attempt"


@mock.patch("wayback_machine_archiver.archiver.time.sleep")
def test_poll_handles_exception_during_status_check(mock_sleep):
    """
    Verify that if checking a job's status raises an exception, the job is
    removed from the pending list to prevent repeated errors.
    """
    mock_client = mock.Mock()
    mock_client.check_status.side_effect = Exception("Network Error")

    pending_jobs = {"job-will-fail": "http://example.com"}

    _poll_pending_jobs(mock_client, pending_jobs)

    # Assertions
    assert not pending_jobs, "Job that caused an exception should be removed"
    mock_client.check_status.assert_called_once_with("job-will-fail")
    mock_sleep.assert_called_once()



import sys
from unittest import mock
import pytest
import logging
from wayback_machine_archiver.archiver import main


# We mock the clients to prevent any actual archiving logic from running.
# This keeps the tests focused purely on the CLI parsing and its direct effects.


@mock.patch("wayback_machine_archiver.archiver.os.getenv", return_value=None)
@mock.patch("wayback_machine_archiver.archiver.LegacyClient")
@mock.patch("wayback_machine_archiver.archiver.logging.basicConfig")
@pytest.mark.parametrize(
    "input_level, expected_level",
    [
        ("info", "INFO"),
        ("DEBUG", "DEBUG"),
        ("warning", "WARNING"),
        ("ErRoR", "ERROR"),
    ],
)
def test_log_level_is_case_insensitive(
    mock_basic_config,
    mock_legacy_client,
    mock_getenv,
    input_level,
    expected_level,
):
    """
    Verify that the --log argument is case-insensitive.
    """
    # Arrange
    sys.argv = ["archiver", "--log", input_level]

    # Act
    main()

    # Assert
    mock_basic_config.assert_called_once_with(level=expected_level, filename=None)


def test_version_action_exits():
    """
    Verify that the --version argument exits the program.
    """
    # Arrange
    sys.argv = ["archiver", "--version"]

    # Act & Assert: argparse's version action calls sys.exit, which raises SystemExit
    with pytest.raises(SystemExit):
        main()


@mock.patch("wayback_machine_archiver.archiver.os.getenv", return_value=None)
@mock.patch("wayback_machine_archiver.archiver.LegacyClient")
@mock.patch("wayback_machine_archiver.archiver.logging.basicConfig")
def test_log_to_file_sets_filename(mock_basic_config, mock_legacy_client, mock_getenv):
    """
    Verify that --log-to-file passes the filename to the logging config.
    """
    # Arrange
    log_file = "archive.log"
    sys.argv = ["archiver", "--log-to-file", log_file]

    # Act
    main()

    # Assert
    mock_basic_config.assert_called_once_with(level=logging.WARNING, filename=log_file)


@pytest.mark.parametrize(
    "creds_exist, user_input, expected_wait",
    [
        (True, 2, 5),
        (True, 10, 10),
        (False, 10, 15),
        (False, 20, 20),
    ],
)
@mock.patch("wayback_machine_archiver.archiver.SPN2Client")
@mock.patch("wayback_machine_archiver.archiver.LegacyClient")
def test_rate_limit_override(
    mock_legacy_client, mock_spn2_client, creds_exist, user_input, expected_wait
):
    """
    Verify that the script enforces the minimum rate-limit wait time.
    """
    # Arrange: Mock getenv to simulate credentials being present or not
    with mock.patch(
        "wayback_machine_archiver.archiver.os.getenv",
        return_value="dummy_key" if creds_exist else None,
    ):
        # Configure the SPN2 client's submit_capture to return None.
        # This prevents the test from entering the infinite polling loop.
        mock_spn2_client.return_value.submit_capture.return_value = None

        sys.argv = ["archiver", "http://test.com", "--rate-limit-wait", str(user_input)]

        # Act
        main()

        # Assert
        if creds_exist:
            mock_spn2_client.return_value.submit_capture.assert_called_once_with(
                "http://test.com", rate_limit_wait=expected_wait
            )
            mock_legacy_client.assert_not_called()
        else:
            # Note: The legacy client gets a pre-formatted URL
            formatted_url = "https://web.archive.org/save/http://test.com"
            mock_legacy_client.return_value.archive.assert_called_once_with(
                formatted_url, rate_limit_wait=expected_wait
            )
            mock_spn2_client.assert_not_called()



from collections import namedtuple
from wayback_machine_archiver.archiver import get_namespace

ELEMENT = namedtuple("Element", "tag")


def test_good_namespace():
    NAMESPACE = "{http://www.sitemaps.org/schemas/sitemap/0.9}"
    test_element = ELEMENT("{namespace}urlset".format(namespace=NAMESPACE))

    assert get_namespace(test_element) == NAMESPACE


def test_no_match_namespace():
    NAMESPACE = ""
    test_element = ELEMENT("{namespace}urlset".format(namespace=NAMESPACE))

    assert get_namespace(test_element) == NAMESPACE
